#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from pyramid.decorator import reify
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface, implementedBy, implementer

from pyams_form.form import AJAXEditForm, ajax_config
from pyams_form.interfaces.form import IFormContextPermissionChecker, IWidgetForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.interfaces import IContentTitle, IInnerPage, IPageHeader
from pyams_skin.interfaces.configuration import IBackOfficeConfiguration, IConfiguration
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces import MANAGE_SYSTEM_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_utils.interfaces.site import ISiteRoot
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm, AdminEditForm
from pyams_zmi.interfaces import IPropertiesEditForm, IZMIConfiguration
from pyams_zmi.interfaces.menu import IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.layer import IAdminLayer

from pyams_zmi import _


@adapter_config(context=(ISiteRoot, IAdminLayer, Interface), provides=IContentTitle)
class SiteRootTitleAdapter(ContextRequestViewAdapter):
    """Site root title adapter"""

    @property
    def title(self):
        return IBackOfficeConfiguration(self.context).title


#
# Configuration properties
#

@adapter_config(context=ISiteRoot, provides=IFormContextPermissionChecker)
class SiteRootPermissionChecker(ContextAdapter):
    """Site root permission checker"""

    edit_permission = MANAGE_SYSTEM_PERMISSION


@viewlet_config(name='properties.menu', context=ISiteRoot, layer=IAdminLayer, manager=ISiteManagementMenu,
                permission=VIEW_SYSTEM_PERMISSION, weight=15)
@viewletmanager_config(name='properties.menu', context=ISiteRoot, layer=IAdminLayer, provides=IPropertiesMenu)
@implementer(IPropertiesMenu)
class PropertiesMenuItem(MenuItem):
    """Properties menu"""

    label = _("Properties")
    icon_class = 'fa-edit'
    url = '#properties.html'


@pagelet_config(name='properties.html', context=ISiteRoot, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@ajax_config(name='properties.json', context=ISiteRoot, layer=IPyAMSLayer)
@implementer(IWidgetForm, IInnerPage, IPropertiesEditForm)
class PropertiesEditForm(AdminEditForm):
    """Properties edit form"""

    legend = _("Update main site properties")
    ajax_handler = 'properties.json'

    @reify
    def fields(self):

        def get_configuration_interface(context):
            for interface in implementedBy(context.__class__).interfaces():
                if issubclass(interface, IConfiguration):
                    return interface

        content = self.getContent()
        return field.Fields(get_configuration_interface(content)).omit('__name__')

    def getContent(self):
        return IConfiguration(self.context)

    def get_ajax_output(self, changes):
        if changes:
            return {
                'status': 'reload',
                'location': '#properties.html',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'
            }
        else:
            return super(self.__class__, self).get_ajax_output(changes)


@adapter_config(context=(Interface, IPyAMSLayer, PropertiesEditForm), provides=IPageHeader)
class PropertiesEditFormHeaderAdapter(DefaultPageHeaderAdapter):
    """Utilities header adapter"""

    @property
    def title(self):
        return IBackOfficeConfiguration(self.context).title

    icon_class = 'fa fa-fw fa-twitch'


#
# Back-office configuration properties
#

@viewlet_config(name='back-office-properties.menu', context=ISiteRoot, layer=IAdminLayer, manager=IPropertiesMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=1)
class BackOfficePropertiesMenuItem(MenuItem):
    """Back-office properties menu"""

    label = _("Back-office properties...")
    icon_class = 'fa-tachometer'
    url = 'back-office-properties.html'
    modal_target = True


@pagelet_config(name='back-office-properties.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class BackOfficePropertiesEditForm(AdminDialogEditForm):
    """Back-office properties edit form"""

    prefix = 'backoffice_properties.'

    legend = _("Update site back-office properties")
    ajax_handler = 'back-office-properties.json'
    edit_permission = MANAGE_SYSTEM_PERMISSION

    dialog_class = 'modal-large'

    @reify
    def fields(self):

        def get_configuration_interface(context):
            for interface in implementedBy(context.__class__).interfaces():
                if issubclass(interface, IBackOfficeConfiguration):
                    return interface

        content = self.getContent()
        return field.Fields(get_configuration_interface(content)).omit('__name__')

    def getContent(self):
        return IBackOfficeConfiguration(self.context)


@view_config(name='back-office-properties.json', context=ISiteRoot, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class BackOfficePropertiesAJAXEditForm(AJAXEditForm, BackOfficePropertiesEditForm):
    """Back-office properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if changes:
            return {
                'status': 'reload',
                'location': '#properties.html',
                'smallbox': self.request.localizer.translate(self.successMessage),
                'smallbox_status': 'success'
            }
        else:
            return AJAXEditForm.get_ajax_output(self, changes)


#
# ZMI configuration properties
#

@viewlet_config(name='zmi-configuration.menu', context=ISiteRoot, layer=IAdminLayer, manager=IPropertiesMenu,
                permission=MANAGE_SYSTEM_PERMISSION, weight=2)
class ZMIConfigurationMenuItem(MenuItem):
    """ZMI configuration menu"""

    label = _("ZMI configuration...")
    icon_class = 'fa-window-restore'
    url = 'zmi-configuration.html'
    modal_target = True


@pagelet_config(name='zmi-configuration.html', context=ISiteRoot, layer=IPyAMSLayer,
                permission=MANAGE_SYSTEM_PERMISSION)
class ZMIConfigurationEditForm(AdminDialogEditForm):
    """ZMI configuration edit form"""

    prefix = 'zmi_properties.'

    legend = _("Update ZMI configuration")
    label_css_class = 'control-label col-md-4'
    input_css_class = 'col-md-8'

    ajax_handler = "zmi-configuration.json"
    edit_permission = MANAGE_SYSTEM_PERMISSION

    fields = field.Fields(IZMIConfiguration)


@view_config(name='zmi-configuration.json', context=ISiteRoot, request_type=IPyAMSLayer,
             permission=MANAGE_SYSTEM_PERMISSION, renderer='json', xhr=True)
class ZMIConfigurationAJAXEditForm(AJAXEditForm, ZMIConfigurationEditForm):
    """ZMI configuration edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        if changes:
            return {'status': 'redirect'}
        else:
            return super(ZMIConfigurationAJAXEditForm, self).get_ajax_output(changes)
